# -*- coding: utf-8 -*-
"""backup module"""

__author__  = "Adrien DELLE CAVE"
__license__ = """
    Copyright (C) 2017-2018  fjord-technologies

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA..
"""

import json
import logging
import os
import requests
import time
import yaml

from datetime import datetime
from dwho.classes.notifiers import DWhoPushNotifications
from dwho.classes.modules import DWhoModuleBase, MODULES
from dwho.config import DWHO_SHARED
from sonicprobe.libs import xys
from sonicprobe.libs.http_json_server import HttpReqErrJson
from sonicprobe.libs.moresynchro import RWLock


requests.packages.urllib3.disable_warnings()


LOG = logging.getLogger('xtrargon.modules.backup')


class XtrargonBackupModule(DWhoModuleBase):
    MODULE_NAME = 'backup'

    LOCK        = RWLock()

    def safe_init(self, options):
        self.server_id     = self.config['general']['server_id']
        self.lock_timeout  = self.config['general']['lock_timeout']
        self.notifications = DWhoPushNotifications(config_path = self.config['general']['notifiers_path'])
        DWHO_SHARED.add('xtrargon')


    NOTIFY_SCHEMA = xys.load("""
    uid!-: !!str
    client_id!-: !!str
    backup!-: !!str
    type!-: !!str
    partial*: !!bool False
    status!-: !~~enum(processing,failed,successful)
    backup_started*-: !!str
    backup_ended*-: !!str
    hostname!-: !!str
    errors*: [ !!str ]
    """)

    def notify(self, request):
        args = request.payload_params()

        if not xys.validate(args, self.NOTIFY_SCHEMA):
            raise HttpReqErrJson(415, "invalid arguments for command")

        if not self.LOCK.acquire_write(self.lock_timeout):
            raise HttpReqErrJson(503, "unable to take LOCK for writing after %s seconds" % self.lock_timeout)

        try:
            if args['type'] not in ('full', 'incr'):
                return

            xvars = args.copy()
            xvars['total_seconds'] = None

            if args.get('backup_started') and args.get('backup_ended'):
                xformat                = '%Y-%m-%d_%H-%M-%S'
                started                = datetime.strptime(args['backup_started'], xformat)
                ended                  = datetime.strptime(args['backup_ended'], xformat)
                xvars['total_seconds'] = (ended - started).total_seconds()

            DWHO_SHARED.set("xtrargon://%s" % args['uid'], 'state', args)

            self.notifications(xvars)

            return xvars
        except HttpReqErrJson:
            raise
        except Exception, e:
            LOG.exception(e)
            raise HttpReqErrJson(503, repr(e))
        finally:
            self.LOCK.release()


if __name__ != "__main__":
    def _start():
        MODULES.register(XtrargonBackupModule())
    _start()
